@@ -17,12 +17,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
 
 
 <chapter>
@@ -175,7 +175,7 @@
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 #import	<Foundation/NSDictionary.h>
 #import	<Foundation/NSEnumerator.h>
-
+#import <Foundation/NSPathUtilities.h>
 #if	defined(__cplusplus)
 extern "C" {
 #endif
@@ -191,6 +191,8 @@ extern "C" {
 @class NSError;
 @class NSURL;
 
+@protocol NSFileManagerDelegate;
+
 /* MacOS-X defines OSType as a 32bit unsigned integer.
  */
 #ifndef OSTYPE_DECLARED
@@ -198,10 +200,22 @@ typedef	uint32_t	OSType;
 #define OSTYPE_DECLARED
 #endif
 
+DEFINE_BLOCK_TYPE(GSDirEnumErrorHandler, BOOL, NSURL*, NSError*);
+  
+enum _NSDirectoryEnumerationOptions
+  {
+    NSDirectoryEnumerationSkipsSubdirectoryDescendants = 1L << 0,
+    NSDirectoryEnumerationSkipsPackageDescendants = 1L << 1,
+    NSDirectoryEnumerationSkipsHiddenFiles = 1L << 2
+  };
+typedef NSUInteger NSDirectoryEnumerationOptions; 
+  
+GS_EXPORT_CLASS
 @interface NSFileManager : NSObject
 {
 #if	GS_EXPOSE(NSFileManager)
 @private
+  id<NSFileManagerDelegate> _delegate;
   NSString	*_lastError;
 #endif
 #if     GS_NONFRAGILE
@@ -221,6 +235,15 @@ typedef	uint32_t	OSType;
  */
 + (NSFileManager*) defaultManager;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+#if GS_HAS_DECLARED_PROPERTIES
+@property (assign) id<NSFileManagerDelegate> delegate;
+#else
+- (id<NSFileManagerDelegate>) delegate;
+- (void) setDelegate: (id<NSFileManagerDelegate>)delegate;
+#endif
+#endif
+
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
 - (NSDictionary *) attributesOfItemAtPath: (NSString*)path
 				    error: (NSError**)error;
@@ -282,6 +305,14 @@ typedef	uint32_t	OSType;
  */
 - (BOOL) removeItemAtURL: (NSURL*)url
                    error: (NSError**)error;
+/**
+ * Creates a symbolic link at the path
+ * that point to the destination path.<br />
+ * Returns YES on success, otherwise NO.
+ */
+- (BOOL) createSymbolicLinkAtPath: (NSString*)path
+              withDestinationPath: (NSString*)destPath
+                            error: (NSError**)error;
 #endif
 
 /**
@@ -299,15 +330,53 @@ typedef	uint32_t	OSType;
 - (BOOL) contentsEqualAtPath: (NSString*)path1
 		     andPath: (NSString*)path2;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+/**
+ * Returns an array of NSURL of the contents of the specified directory. <br />
+ * The listing is shallow and does not recurse into subdirectories.
+ * The special files '.' and '..' are excluded but it can return
+ * hidden files.<br />
+ * The only mask option supported is
+ * NSDirectoryEnumerationSkipsHiddenFiles.<br />
+ * The current implementation handles only files and property keys are ignored.
+ */
+- (NSArray*) contentsOfDirectoryAtURL: (NSURL*)url
+           includingPropertiesForKeys: (NSArray*)keys
+                              options: (NSDirectoryEnumerationOptions)mask
+                                error: (NSError **)error;
+
+/**
+ * Locates and, optionally, creates the specified common directory in
+ * domain
+ */
+- (NSURL *)URLForDirectory: (NSSearchPathDirectory)directory 
+                  inDomain: (NSSearchPathDomainMask)domain 
+         appropriateForURL: (NSURL *)url 
+                    create: (BOOL)shouldCreate 
+                     error: (NSError **)error;
+
+/**
+ * Enumerate over the contents of a directory.
+ */
+- (NSDirectoryEnumerator *)enumeratorAtURL: (NSURL *)url
+                includingPropertiesForKeys: (NSArray *)keys 
+                                   options: (NSDirectoryEnumerationOptions)mask 
+                              errorHandler: (GSDirEnumErrorHandler)handler;
+#endif
+  
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
 /**
- * Returns an array of the contents of the specified directory.<br />
+ * Returns an array of NSStrings of the contents of the
+ * specified directory.<br />
  * The listing does <strong>not</strong> recursively list subdirectories.<br />
  * The special files '.' and '..' are not listed.<br />
  * Indicates an error by returning nil (eg. if path is not a directory or
  * it can't be read for some reason).
  */
 - (NSArray*) contentsOfDirectoryAtPath: (NSString*)path error: (NSError**)error;
+
+- (NSDictionary*) attributesOfFileSystemForPath: (NSString*)path
+                                          error: (NSError**)error;
 #endif
 
 - (BOOL) copyPath: (NSString*)source
@@ -343,6 +412,11 @@ typedef	uint32_t	OSType;
  * </p>
  */
 - (NSDirectoryEnumerator*) enumeratorAtPath: (NSString*)path;
+
+/** Returns the attributes dictionary for the file at the specified path.
+ * If that file is a symbolic link, the flag determines whether the attributes
+ * returned are those of the link or those of the destination file.
+ */
 - (NSDictionary*) fileAttributesAtPath: (NSString*)path
 			  traverseLink: (BOOL)flag;
 
@@ -419,6 +493,7 @@ typedef	uint32_t	OSType;
  * if they wish to deal with copy and move operations performed
  * by NSFileManager.
  */
+
 @interface NSObject (NSFileManagerHandler)
 /**
  * <p>When an error occurs during a copy or move operation, the file manager
@@ -470,7 +545,6 @@ typedef	uint32_t	OSType;
      willProcessPath: (NSString*)path;
 @end
 
-
 /**
  *  <p>This is a subclass of <code>NSEnumerator</code> which provides a full
  *  listing of all the files beneath a directory and its subdirectories.
@@ -484,6 +558,7 @@ typedef	uint32_t	OSType;
  *  and in the current implementation the natural order of the underlying
  *  filesystem is used.</p>
  */
+GS_EXPORT_CLASS
 @interface NSDirectoryEnumerator : NSEnumerator
 {
 #if	GS_EXPOSE(NSDirectoryEnumerator)
@@ -492,11 +567,13 @@ typedef	uint32_t	OSType;
   NSString *_topPath;
   NSString *_currentFilePath;
   NSFileManager *_mgr;
-  struct 
+  GSDirEnumErrorHandler _errorHandler; 
+  struct _NSDirectoryEnumeratorFlags      // tag for objc++ w/gcc 4.6 
   {
     BOOL isRecursive: 1;
     BOOL isFollowing: 1;
     BOOL justContents: 1;
+    BOOL skipHidden: 1;
   } _flags;
 #endif
 #if     GS_NONFRAGILE
@@ -634,6 +711,77 @@ GS_EXPORT NSString* const NSFileSystemFreeNodes;
 - (NSUInteger) fileSystemFileNumber;
 @end
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST)
+
+@protocol NSFileManagerDelegate <NSObject>
+#if GS_PROTOCOLS_HAVE_OPTIONAL
+@optional
+#else
+@end
+@interface NSObject (NSFileManagerDelegate)
+#endif
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldCopyItemAtPath: (NSString *)srcPath
+                toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldCopyItemAtURL: (NSURL *)srcURL
+                toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+        copyingItemAtPath: (NSString *)srcPath
+                   toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+         copyingItemAtURL: (NSURL *)srcURL
+                    toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldMoveItemAtPath: (NSString *)srcPath
+                toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldMoveItemAtURL: (NSURL *)srcURL
+                toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+         movingItemAtPath: (NSString *)srcPath
+                   toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+          movingItemAtURL: (NSURL *)srcURL
+                    toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldLinkItemAtPath: (NSString *)srcPath
+                toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldLinkItemAtURL: (NSURL *)srcURL
+                toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+        linkingItemAtPath: (NSString *)srcPath
+                   toPath: (NSString *)dstPath;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+         linkingItemAtURL: (NSURL *)srcURL
+                    toURL: (NSURL *)dstURL;
+
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldRemoveItemAtPath: (NSString *)path;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldRemoveItemAtURL: (NSURL *)URL;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+       removingItemAtPath: (NSString *)path;
+- (BOOL)fileManager: (NSFileManager *)fileManager
+  shouldProceedAfterError: (NSError *)error
+        removingItemAtURL: (NSURL *)URL;
+@end
+
+#endif
+
 #if	defined(__cplusplus)
 }
 #endif
