@@ -14,19 +14,19 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */ 
 
 #ifndef __NSHTTPCookieStorage_h_GNUSTEP_BASE_INCLUDE
 #define __NSHTTPCookieStorage_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION(011300,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_2,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
 
@@ -38,39 +38,42 @@ extern "C" {
 @class NSHTTPCookie;
 @class NSURL;
 
+enum {
+  NSHTTPCookieAcceptPolicyAlways,
+  NSHTTPCookieAcceptPolicyNever,
+  NSHTTPCookieAcceptPolicyOnlyFromMainDocumentDomain
+};
 /**
  * NSHTTPCookieAcceptPolicyAlways Accept all cookies
  * NSHTTPCookieAcceptPolicyNever Reject all cookies
  * NSHTTPCookieAcceptPolicyOnlyFromMainDocumentDomain Accept cookies
  * only from the main document domain
  */
-typedef enum {
-  NSHTTPCookieAcceptPolicyAlways,
-  NSHTTPCookieAcceptPolicyNever,
-  NSHTTPCookieAcceptPolicyOnlyFromMainDocumentDomain
-} NSHTTPCookieAcceptPolicy;
+typedef NSUInteger NSHTTPCookieAcceptPolicy;
 
 /**
  * Posted to the distributed notification center when the cookie
  * accept policy is changed.
  */
-extern NSString * const NSHTTPCookieManagerAcceptPolicyChangedNotification;
+GS_EXPORT NSString * const NSHTTPCookieManagerAcceptPolicyChangedNotification;
 
 /**
  * Posted when the set of cookies changes
  */
-extern NSString * const NSHTTPCookieManagerCookiesChangedNotification;
+GS_EXPORT NSString * const NSHTTPCookieManagerCookiesChangedNotification;
 
 
 /**
  * The NSHTTPCookieStorage class provides a shared instance which handles
  * the shared cookie store.<br />
  */
-
+GS_EXPORT_CLASS
 @interface NSHTTPCookieStorage :  NSObject
 {
+#if	GS_EXPOSE(NSHTTPCookieStorage)
 @private
   void	*_NSHTTPCookieStorageInternal;
+#endif
 }
 
 /**
