@@ -14,12 +14,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
 
    AutogsdocSource: NSIndexPath.m
 
@@ -35,13 +35,14 @@
 extern "C" {
 #endif
 
-#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION( 10200,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST) && GS_API_VERSION( 10200,GS_API_LATEST)
 
 /**
  * Instances of this class represent a series of indexes into a hierarchy
  * of arrays.<br />
  * Each instance is a unique shared object.
  */
+GS_EXPORT_CLASS
 @interface	NSIndexPath : NSObject <NSCopying, NSCoding>
 {
 #if	GS_EXPOSE(NSIndexPath)
@@ -50,8 +51,14 @@ extern "C" {
   NSUInteger	_length;
   NSUInteger	*_indexes;
 #endif
-#if	!GS_NONFRAGILE
-  void		*_unused;
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
 #endif
 }
 
