@@ -14,12 +14,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */ 
 
 #ifndef __NSProtocolChecker_h_GNUSTEP_BASE_INCLUDE
@@ -35,6 +35,7 @@ extern "C" {
 
 @class Protocol;
 
+GS_EXPORT_CLASS
 @interface NSProtocolChecker : NSProxy
 {
 #if	GS_EXPOSE(NSProtocolChecker)
@@ -42,8 +43,14 @@ extern "C" {
   Protocol *_myProtocol;
   NSObject *_myTarget;
 #endif
-#if	!GS_NONFRAGILE
-  void	*_unused;
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
 #endif
 }
 
