@@ -14,12 +14,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
   */
 
 /**
@@ -41,14 +41,20 @@
  The mode defaults to 'gnustep' but may be set using the GSPathHandling()
  function.<br />
  You should probably stick to using the default 'gnustep' mode in which the
- path handling methods cope with both unix and windows style paths in
+ path handling methods cope with both 'unix' and 'windows' style paths in
  portable and tolerant manner:<br />
  Paths are read in literally so they can be in the native format provided
- by the operating system or in a non-native format.<br />
+ by the operating system or in a non-native format. See
+ [NSFileManager-stringWithFileSystemRepresentation:length:].<br />
  Paths are written out using the native format of the system the application
- is running on (eg on windows slashes are converted to backslashes).<br />
+ is running on (eg on windows slashes are converted to backslashes).
+ See [NSFileManager-fileSystemRepresentationWithPath:].<br />
  The path handling methods accept either a forward or backward slash as a
  path separator when parsing any path.<br />
+ Unless operating in 'unix' mode, a leading letter followed by a colon is
+ considered the start of a windows style path (the drive specifier), and a
+ path beginning with something of the form '//host/share/' is considered
+ the start of a UNC style path.<br />
  The path handling methods add forward slashes when building new paths
  internally or when standardising paths, so those path strings provide
  a portable representation (as long as they are relative paths, not including
@@ -56,8 +62,23 @@
  An important case to note is that on windows a path which looks at first
  glance like an absolute path may actually be a relative one.<br />
  'C:file' is a relative path because it specifies  a file on the C drive
- but does not say what directory it is in.
+ but does not say what directory it is in.<br />
+Similarly, '/dir/file' is a relative path because it specifies the full
+location fo a file on a drive, but does not specify which drive it is on.
  </p>
+<p>As a consequence of this path handling, you are able to work completely
+portably using relative paths (adding components, extensions and
+relative paths to a pth, or removing components, extensions and relative
+paths from a path etc), and when you save paths as strings in files
+which may be transferred to another platform, you should save a relative
+path.<br />
+When you need to know absolute paths of various points in the filesystem,
+you can use various path utility functions to obtain those absolute paths.
+For instance, instead of saving an absolute path to a file, you might want
+to save a path relative to a user's home directory.  You could do that by
+calling NSHomeDirectory() to get the home directory, and only saving the
+part of the full path after that prefix.
+</p>
 </chapter>
  */ 
 
@@ -77,7 +98,7 @@ extern "C" {
  */
 typedef uint16_t unichar;
 
-#if OS_API_VERSION(100500,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST) 
 #define NSMaximumStringLength   (INT_MAX-1)
 #endif
 
@@ -100,12 +121,23 @@ enum
   NSBackwardsSearch = 4,
   NSAnchoredSearch = 8,
   NSNumericSearch = 64	/* MacOS-X 10.2 */
-#if OS_API_VERSION(100500,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST) 
  ,
  NSDiacriticInsensitiveSearch = 128,
  NSWidthInsensitiveSearch = 256,
  NSForcedOrderingSearch = 512
 #endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_7,GS_API_LATEST) 
+ ,
+ /**
+  * Treats the search string as a regular expression.  This option may be
+  * combined with NSCaseInsensitiveSearch and NSAnchoredSearch, but no other
+  * search options.
+  *
+  * This option may only be used with the -rangeOfString: family of methods.
+  */
+ NSRegularExpressionSearch = 1024
+#endif
 };
 typedef NSUInteger NSStringCompareOptions;
 
@@ -181,7 +213,7 @@ typedef enum _NSStringEncoding
   NSBIG5StringEncoding,			// Traditional chinese
   NSKoreanEUCStringEncoding		// Korean
 
-#if OS_API_VERSION(100400,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST) 
   ,
   NSUTF16BigEndianStringEncoding = 0x90000100,
   NSUTF16LittleEndianStringEncoding = 0x94000100,
@@ -195,7 +227,7 @@ enum {
   NSOpenStepUnicodeReservedBase = 0xF400
 };
 
-#if OS_API_VERSION(100400,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST) 
 enum {
   NSStringEncodingConversionAllowLossy = 1,
   NSStringEncodingConversionExternalRepresentation = 2
@@ -247,24 +279,26 @@ typedef NSUInteger NSStringEncodingConversionOptions;
  *   that of the receiver which returned them.
  * </p>
  */
+
+GS_EXPORT_CLASS
 @interface NSString :NSObject <NSCoding, NSCopying, NSMutableCopying>
 
 + (id) string;
 + (id) stringWithCharacters: (const unichar*)chars
 		     length: (NSUInteger)length;
-#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION( 10200,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST) && GS_API_VERSION( 10200,GS_API_LATEST)
 + (id) stringWithCString: (const char*)byteString
 		encoding: (NSStringEncoding)encoding;
 #endif
 + (id) stringWithCString: (const char*)byteString
 		  length: (NSUInteger)length;
 + (id) stringWithCString: (const char*)byteString;
-+ (id) stringWithFormat: (NSString*)format,...;
++ (id) stringWithFormat: (NSString*)format, ... NS_FORMAT_FUNCTION(1,2);
 + (id) stringWithContentsOfFile:(NSString *)path;
 
 // Initializing Newly Allocated Strings
 - (id) init;
-#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION( 10200,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST) && GS_API_VERSION( 10200,GS_API_LATEST)
 - (id) initWithBytes: (const void*)bytes
 	      length: (NSUInteger)length
 	    encoding: (NSStringEncoding)encoding;
@@ -273,7 +307,7 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 		  encoding: (NSStringEncoding)encoding 
 	      freeWhenDone: (BOOL)flag;
 #endif
-#if OS_API_VERSION(100400,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST)
 + (id) stringWithContentsOfFile: (NSString*)path
                    usedEncoding: (NSStringEncoding*)enc
                           error: (NSError**)error;
@@ -307,13 +341,15 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 	   encoding: (NSStringEncoding)enc
 	      error: (NSError**)error;
 #endif
-#if OS_API_VERSION(100500,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST)
 - (NSString*)stringByReplacingOccurrencesOfString: (NSString*)replace
                                        withString: (NSString*)by
                                           options: (NSStringCompareOptions)opts
                                             range: (NSRange)searchRange;
 - (NSString*)stringByReplacingOccurrencesOfString: (NSString*)replace
                                        withString: (NSString*)by;
+- (NSString*) stringByReplacingCharactersInRange: (NSRange)aRange 
+                                      withString: (NSString*)by;
 #endif
 - (id) initWithCharactersNoCopy: (unichar*)chars
 			 length: (NSUInteger)length
@@ -327,13 +363,12 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 	        length: (NSUInteger)length;
 - (id) initWithCString: (const char*)byteString;
 - (id) initWithString: (NSString*)string;
-- (id) initWithFormat: (NSString*)format, ...;
+- (id) initWithFormat: (NSString*)format, ... NS_FORMAT_FUNCTION(1,2);
 - (id) initWithFormat: (NSString*)format
-	    arguments: (va_list)argList;
+	    arguments: (va_list)argList NS_FORMAT_FUNCTION(1,0);
 - (id) initWithData: (NSData*)data
 	   encoding: (NSStringEncoding)encoding;
 - (id) initWithContentsOfFile: (NSString*)path;
-- (id) init;
 
 // Getting a String's Length
 - (NSUInteger) length;
@@ -345,13 +380,13 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 		 range: (NSRange)aRange;
 
 // Combining Strings
-- (NSString*) stringByAppendingFormat: (NSString*)format,...;
+- (NSString*) stringByAppendingFormat: (NSString*)format, ...
+  NS_FORMAT_FUNCTION(1,2);
 - (NSString*) stringByAppendingString: (NSString*)aString;
 
 // Dividing Strings into Substrings
 - (NSArray*) componentsSeparatedByString: (NSString*)separator;
 - (NSString*) substringFromIndex: (NSUInteger)index;
-- (NSString*) substringFromRange: (NSRange)aRange;
 - (NSString*) substringToIndex: (NSUInteger)index;
 
 // Finding Ranges of Characters and Substrings
@@ -371,8 +406,26 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 // Determining Composed Character Sequences
 - (NSRange) rangeOfComposedCharacterSequenceAtIndex: (NSUInteger)anIndex;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_2,GS_API_LATEST) 
+/** Returns a copy of the receiver normalised using the KD form.
+ */
+- (NSString *) decomposedStringWithCompatibilityMapping;
+
+/** Returns a copy of the receiver normalised using the D form.
+ */
+- (NSString *) decomposedStringWithCanonicalMapping;
+
+/** Returns a copy of the receiver normalised using the KC form.
+ */
+- (NSString *) precomposedStringWithCompatibilityMapping;
+
+/** Returns a copy of the receiver normalised using the C form.
+ */
+- (NSString *) precomposedStringWithCanonicalMapping;
+#endif
+
 // Converting String Contents into a Property List
-- (id)propertyList;
+- (id) propertyList;
 - (NSDictionary*) propertyListFromStringsFileFormat;
 
 // Identifying and Comparing Strings
@@ -401,7 +454,7 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 - (const char*) cString;
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 
-#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION( 10200,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST) && GS_API_VERSION( 10200,GS_API_LATEST)
 - (const char*) cStringUsingEncoding: (NSStringEncoding)encoding;
 - (BOOL) getCString: (char*)buffer
 	  maxLength: (NSUInteger)maxLength
@@ -544,6 +597,10 @@ typedef NSUInteger NSStringEncodingConversionOptions;
  *   @"/" with @"/file" produces @"/file"
  *   @"path with @"C:/file" produces @"path/file"
  * </example>
+ * NB. Do not use this method to modify strings other than filesystem
+ * paths as the behavior in such cases is undefined ... for instance
+ * the string may have repeated slashes or slash-dot-slash sequences
+ * removed.
  */
 - (NSString*) stringByAppendingPathComponent: (NSString*)aString;
 
@@ -565,6 +622,10 @@ typedef NSUInteger NSStringEncodingConversionOptions;
  *   @"/" with @"app" produces @"/" (no file name to append to)
  *   @"" with @"app" produces @"" (no file name to append to)
  * </example>
+ * NB. Do not use this method to modify strings other than filesystem
+ * paths as the behavior in such cases is undefined ... for instance
+ * the string may have repeated slashes or slash-dot-slash sequences
+ * removed.
  */
 - (NSString*) stringByAppendingPathExtension: (NSString*)aString;
 
@@ -587,6 +648,10 @@ typedef NSUInteger NSStringEncodingConversionOptions;
  *   @"//host/share/" produces @"//host/share/" (a UNC path)
  *   @"//path/file" produces @"//path" (an absolute Unix path)
  * </example>
+ * NB. Do not use this method to modify strings other than filesystem
+ * paths as the behavior in such cases is undefined ... for instance
+ * the string may have repeated slashes or slash-dot-slash sequences
+ * removed.
  */
 - (NSString*) stringByDeletingLastPathComponent;
 
@@ -607,6 +672,10 @@ typedef NSUInteger NSStringEncodingConversionOptions;
  *   @"/.ext" produces @"/.ext" (there is no file to strip from)
  *   @".ext" produces @".ext" (there is no file to strip from)
  * </example>
+ * NB. Do not use this method to modify strings other than filesystem
+ * paths as the behavior in such cases is undefined ... for instance
+ * the string may have repeated slashes or slash-dot-slash sequences
+ * removed.
  */
 - (NSString*) stringByDeletingPathExtension;
 
@@ -620,8 +689,10 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 - (NSString*) stringByExpandingTildeInPath;
 
 /**
+ * First calls -stringByExpandingTildeInPath if necessary.<br />
  * Replaces path string by one in which path components representing symbolic
  * links have been replaced by their referents.<br />
+ * Removes a leading '/private' if the result is valid.<br />
  * If links cannot be resolved, returns an unmodified copy of the receiver.
  */
 - (NSString*) stringByResolvingSymlinksInPath;
@@ -691,16 +762,17 @@ typedef NSUInteger NSStringEncodingConversionOptions;
  */
 - (NSArray*) stringsByAppendingPaths: (NSArray*)paths;
 
-+ (NSString*) localizedStringWithFormat: (NSString*)format, ...;
++ (NSString*) localizedStringWithFormat: (NSString*)format, ...
+  NS_FORMAT_FUNCTION(1,2);
 
 + (id) stringWithString: (NSString*)aString;
 + (id) stringWithContentsOfURL: (NSURL*)url;
 + (id) stringWithUTF8String: (const char*)bytes;
 - (id) initWithFormat: (NSString*)format
-	       locale: (NSDictionary*)locale, ...;
+	       locale: (NSDictionary*)locale, ... NS_FORMAT_FUNCTION(1,3);
 - (id) initWithFormat: (NSString*)format
 	       locale: (NSDictionary*)locale
-	    arguments: (va_list)argList;
+	    arguments: (va_list)argList NS_FORMAT_FUNCTION(1,0);
 - (id) initWithUTF8String: (const char *)bytes;
 - (id) initWithContentsOfURL: (NSURL*)url;
 - (NSString*) substringWithRange: (NSRange)aRange;
@@ -708,7 +780,7 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 - (NSComparisonResult) compare: (NSString*)string 
 		       options: (NSUInteger)mask 
 			 range: (NSRange)compareRange 
-			locale: (NSDictionary*)dict;
+			locale: (id)locale;
 - (NSComparisonResult) localizedCompare: (NSString *)string;
 - (NSComparisonResult) localizedCaseInsensitiveCompare: (NSString *)string;
 - (BOOL) writeToFile: (NSString*)filename
@@ -732,17 +804,22 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 - (const char *)UTF8String;
 #endif
 
-#if OS_API_VERSION(100300,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_9,GS_API_LATEST)
+- (NSString *) stringByAddingPercentEncodingWithAllowedCharacters: (NSCharacterSet *)aSet;
+- (NSString *) stringByRemovingPercentEncoding;
+#endif
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3,GS_API_LATEST) 
 /** Not implemented */
-- (void) getParagraphStart: (NSUInteger *)startPtr
-                       end: (NSUInteger *)parEndPtr
-               contentsEnd: (NSUInteger *)contentsEndPtr
+- (void) getParagraphStart: (NSUInteger *)startIndex
+                       end: (NSUInteger *)parEndIndex
+               contentsEnd: (NSUInteger *)contentsEndIndex
                  forRange: (NSRange)range;
 /** Not implemented */
  - (NSRange) paragraphRangeForRange: (NSRange)range;
 #endif
 
-#if OS_API_VERSION(100500,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST) 
 /**
  * Returns YES when scanning the receiver's text from left to right
  * finds an initial digit in the range 1-9 or a letter in the set
@@ -765,12 +842,22 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 
 #endif
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_10,GS_API_LATEST) 
+
+/**
+  * Returns YES if the receiver contains string, otherwise, NO.
+  */
+- (BOOL) containsString: (NSString *)string;
+
+#endif
+
 #if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 + (Class) constantStringClass;
 #endif	/* GS_API_NONE */
 
 @end
 
+GS_EXPORT_CLASS
 @interface NSMutableString : NSString
 
 // Creating Temporary Strings
@@ -780,7 +867,7 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 + (id) stringWithCString: (const char*)byteString
 		  length: (NSUInteger)length;
 + (id) stringWithCString: (const char*)byteString;
-+ (id) stringWithFormat: (NSString*)format,...;
++ (id) stringWithFormat: (NSString*)format, ... NS_FORMAT_FUNCTION(1,2);
 + (id) stringWithContentsOfFile: (NSString*)path;
 + (NSMutableString*) stringWithCapacity: (NSUInteger)capacity;
 
@@ -788,7 +875,7 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 - (id) initWithCapacity: (NSUInteger)capacity;
 
 // Modify A String
-- (void) appendFormat: (NSString*)format, ...;
+- (void) appendFormat: (NSString*)format, ... NS_FORMAT_FUNCTION(1,2);
 - (void) appendString: (NSString*)aString;
 - (void) deleteCharactersInRange: (NSRange)range;
 - (void) insertString: (NSString*)aString atIndex: (NSUInteger)loc;
@@ -802,6 +889,12 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 
 @end
 
+#ifdef __OBJC_GNUSTEP_RUNTIME_ABI__
+#  if __OBJC_GNUSTEP_RUNTIME_ABI__ >= 20
+#    define GNUSTEP_NEW_STRING_ABI
+#  endif
+#endif
+
 /**
  * <p>The NXConstantString class is used to hold constant 8-bit character
  * string objects produced by the compiler where it sees @"..." in the
@@ -826,10 +919,48 @@ typedef NSUInteger NSStringEncodingConversionOptions;
  * What follows is a dummy declaration of the class to keep the compiler
  * happy.
  */
+
+GS_EXPORT_CLASS
 @interface NXConstantString : NSString
 {
+@public
+#ifdef GNUSTEP_NEW_STRING_ABI
+  /**
+   * Flags.  The low 16 bits are reserved for the compiler, the top 16 for use
+   * by the Foundation Framework.  Currently only the low 2 bits are used, to
+   * indicate the encoding of the string, with the following values:
+   *
+   * 0. ASCII (UTF-8 using only 7-bit characters)
+   * 1. UTF-8
+   * 2. UTF-16
+   * 3. UTF-32
+   *
+   */
+  uint32_t flags;
+  /**
+   * The number of characters (UTF-16 code units) in the string.
+   */
+  uint32_t nxcslen;
+  /**
+   * The number of bytes in the string.  For fixed-length encodings, this is a
+   * fixed multiple of nxcslen, but for UTF-8 it can be different.
+   */
+  uint32_t size;
+  /**
+   * Hash value.
+   */
+  uint32_t hash;
+  /**
+   * Pointer to the byte data of the string.  Note that `char*` is the correct
+   * type only if the low two bits of the flags indicate that this is an ASCII
+   * or UTF-8 string, otherwise it is a pointer to 16- or 32-bit characters in
+   * native byte order.
+   */
+  const char * const nxcsptr;
+#else
   const char * const nxcsptr;
   const unsigned int nxcslen;
+#endif
 }
 @end
 
