@@ -14,12 +14,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */ 
 
 #ifndef __NSTimer_h_GNUSTEP_BASE_INCLUDE
@@ -28,6 +28,9 @@
 
 #import	<Foundation/NSDate.h>
 
+@class NSTimer;
+DEFINE_BLOCK_TYPE(GSTimerBlock, void, NSTimer*);
+
 #if	defined(__cplusplus)
 extern "C" {
 #endif
@@ -39,20 +42,28 @@ extern "C" {
  *	Other classes must not attempt to use instance variables as
  *	they are subject to change.
  */
+GS_EXPORT_CLASS
 @interface NSTimer : NSObject
 {
 #if	GS_EXPOSE(NSTimer)
 @public
-  NSDate 	*_date;		/* Must be first - for NSRunLoop optimisation */
-  BOOL		_invalidated;	/* Must be 2nd - for NSRunLoop optimisation */
-  BOOL		_repeats;
+  NSDate 	 *_date;	/* Must be 1st - for NSRunLoop optimisation */
+  BOOL		 _invalidated;	/* Must be 2nd - for NSRunLoop optimisation */
+  BOOL		 _repeats;
   NSTimeInterval _interval;
-  id		_target;
-  SEL		_selector;
-  id		_info;
+  id		 _target;
+  SEL		 _selector;
+  id		 _info;
+  GSTimerBlock   _block;
 #endif
-#if	!GS_NONFRAGILE
-  void		*_unused;
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
 #endif
 }
 
@@ -61,21 +72,31 @@ extern "C" {
 + (NSTimer*) scheduledTimerWithTimeInterval: (NSTimeInterval)ti
 				 invocation: (NSInvocation*)invocation
 				    repeats: (BOOL)f;
+  
 + (NSTimer*) scheduledTimerWithTimeInterval: (NSTimeInterval)ti
 				     target: (id)object
 				   selector: (SEL)selector
 				   userInfo: (id)info
 				    repeats: (BOOL)f;
-
+  
++ (NSTimer *) scheduledTimerWithTimeInterval: (NSTimeInterval)ti
+                                     repeats: (BOOL)f
+                                       block: (GSTimerBlock)block;
+  
 + (NSTimer*) timerWithTimeInterval: (NSTimeInterval)ti
 		        invocation: (NSInvocation*)invocation
 			   repeats: (BOOL)f;
+
 + (NSTimer*) timerWithTimeInterval: (NSTimeInterval)ti
 			    target: (id)object
 			  selector: (SEL)selector
 			  userInfo: (id)info
 			   repeats: (BOOL)f;
 
++ (NSTimer*) timerWithTimeInterval: (NSTimeInterval)ti
+			   repeats: (BOOL)f
+			     block: (GSTimerBlock)block;
+
 - (void) fire;
 - (NSDate*) fireDate;
 - (void) invalidate;
@@ -86,7 +107,7 @@ extern "C" {
 - (NSTimeInterval) timeInterval;
 #endif
 
-#if	OS_API_VERSION(100200, GS_API_LATEST)
+#if	OS_API_VERSION(MAC_OS_X_VERSION_10_2, GS_API_LATEST)
 - (id) initWithFireDate: (NSDate*)fd
 	       interval: (NSTimeInterval)ti
 		 target: (id)object
@@ -96,6 +117,13 @@ extern "C" {
 - (void) setFireDate: (NSDate*)fireDate;
 #endif
 
+#if	OS_API_VERSION(MAC_OS_X_VERSION_10_12, GS_API_LATEST)  
+- (instancetype) initWithFireDate: (NSDate *)date 
+                         interval: (NSTimeInterval)interval 
+                          repeats: (BOOL)repeats 
+                            block: (GSTimerBlock)block;
+#endif
+  
 @end
 
 #if	defined(__cplusplus)
