@@ -14,21 +14,22 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */ 
 
 #ifndef __NSURLConnection_h_GNUSTEP_BASE_INCLUDE
 #define __NSURLConnection_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_2,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
+#import <Foundation/NSRunLoop.h>
 
 #if	defined(__cplusplus)
 extern "C" {
@@ -43,6 +44,7 @@ extern "C" {
 
 /**
  */
+GS_EXPORT_CLASS
 @interface NSURLConnection : NSObject
 {
 #if	GS_EXPOSE(NSURLConnection)
@@ -66,11 +68,27 @@ extern "C" {
 + (NSURLConnection *) connectionWithRequest: (NSURLRequest *)request
 				   delegate: (id)delegate;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+/**
+ * Start the asynchronous load.  This method is only needed if NO is passed 
+ * into startImmediately when calling initWithRequest: delegate: startImmediately.
+ */ 
+- (void) start;
+#endif
+  
 /**
  * Cancel the asynchronous load in progress (if any) for this connection.
  */
 - (void) cancel;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+- (void) scheduleInRunLoop: (NSRunLoop *)aRunLoop 
+                   forMode: (NSRunLoopMode)mode;
+
+- (void) unscheduleFromRunLoop: (NSRunLoop *)aRunLoop 
+                       forMode: (NSRunLoopMode)mode;
+#endif
+  
 /** <init />
  * Initialises the receiver with the specified request (performing
  * a deep copy so that the request does not change during loading)
@@ -86,6 +104,24 @@ extern "C" {
  */
 - (id) initWithRequest: (NSURLRequest *)request delegate: (id)delegate;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+/** <init />
+ * Initialises the receiver with the specified request (performing
+ * a deep copy so that the request does not change during loading)
+ * and delegate.<br />
+ * This automatically initiates an asynchronous load for the request 
+ * if and only if startImmediately is set to YES.<br />
+ * Processing of the request is done in the thread which calls this
+ * method, so the thread must run its current run loop
+ * (in NSDefaultRunLoopMode) for processing to continue/complete.<br />
+ * The delegate will receive callbacks informing it of the progress
+ * of the load.<br />
+ * This method breaks with convention and retains the delegate object,
+ * releasing it when the connection finished loading, fails, or is cancelled.
+ */
+- (id) initWithRequest: (NSURLRequest *)request delegate: (id)delegate startImmediately: (BOOL)startImmediately;
+#endif
+  
 @end
 
 
@@ -142,7 +178,19 @@ extern "C" {
  *   </item>
  * </list>
  */
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_7,GS_API_LATEST) && GS_API_VERSION(11300,GS_API_LATEST)
+@protocol NSURLConnectionDelegate <NSObject>
+
+#if GS_PROTOCOLS_HAVE_OPTIONAL
+@optional
+#else
+@end
 @interface NSObject (NSURLConnectionDelegate)
+#endif
+
+#else
+@interface NSObject (NSURLConnectionDelegate)
+#endif
 
 /**
  * Instructs the delegate that authentication for challenge has
@@ -211,11 +259,8 @@ extern "C" {
 - (NSURLRequest *) connection: (NSURLConnection *)connection
 	      willSendRequest: (NSURLRequest *)request
 	     redirectResponse: (NSURLResponse *)response;
-
 @end
 
-
-
 /**
  * An interface to perform synchronous loading of URL requests.
  */
