@@ -14,19 +14,19 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */ 
 
 #ifndef __NSURLResponse_h_GNUSTEP_BASE_INCLUDE
 #define __NSURLResponse_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100200,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_2,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
 
@@ -44,6 +44,7 @@ extern "C" {
 /**
  * The response to an NSURLRequest
  */
+GS_EXPORT_CLASS
 @interface NSURLResponse :  NSObject <NSCoding, NSCopying>
 {
 #if	GS_EXPOSE(NSURLResponse)
@@ -67,7 +68,7 @@ extern "C" {
   expectedContentLength: (NSInteger)length
   textEncodingName: (NSString *)name;
 
-#if OS_API_VERSION(100700,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_7,GS_API_LATEST)
 /**
  * Initialises the receiver with the URL, statusCode, HTTPVersion, and
  * headerFields provided.
@@ -114,6 +115,7 @@ extern "C" {
 /**
  * HTTP specific additions to an NSURLResponse
  */
+GS_EXPORT_CLASS
 @interface NSHTTPURLResponse :  NSURLResponse
 
 /**
